%  FMRLAB_model_select() - estimation of model number contained in
%                          the observed fMRI data.
% 
% Usage:
%   >> comp_no = FMRLAB_model_select(met, residual_var);
%
% Inputs:
%   met = method (1=BIC (default), 2=empirical)
%   residual_var = residual variance accepted (default = 0.01)  
% 
% Outputs: 
%   comp_no = estimated component model number
%
% See also:
%   runica(), binica()
%
% Author: 
%   Jeng-Ren Duann, SCCN/INC/UCSD & CNL/Salk Inst. 2004-05
%
% Note:
%   1. this program use binica() with PCA option set to the PCA
%   component number every time for the test and maxsteps option
%   set to zero to get the effect with PCA only.
%   2. the variance account for versus the component number will be
%   plot.
%   3. the optimal component number will be determined
%   automatically with return value, comp_no.

function comp_no = FMRLAB_model_select(residual_var)
  

  global FMRI

  if isempty(FMRI),
    fmrlab_messagedlg('No FMRI data set available',...
      'Error',...
      'Please load FMRI data first before launch this function',...
      {'OK'});
    return
  end

  
  RGB_mainwin = [1 .984 .895];
  RGB_background = [0.860 0.824 0.618];
  RGB_button = [0.860 0.824 0.618]; 
  RGB_text = [0.333 0.283 0.003];
  
  if nargin < 1,
    met = 1;
  end

% creating the user interface to display the PVAF;
  fig = figure(...
      'name','FMRLAB Model Selection',...
      'color',RGB_mainwin,...
      'menubar','none');

if met == 1,
  [mea,comp_no,min_mea]=model_selection(FMRI.data,'BIC');
  plot(mea);
  hold on
  plot(comp_no,min_mea,'ro');
else
  if nargin < 2,
    residual_var = 0.01;
  end
  
  % calculating the PVAF per different component number
  % first run: doing comp_idx = 10, 20, 30,...
  comp_idx = [];
  v_idx = [];
  cnt = 0;
  lowerstep = round(-0.1*size(FMRI.data,1));
  for i=size(FMRI.data,1):lowerstep:10
    [w,sph] = binica(FMRI.data,'pca',i,'maxsteps',0);
    W = w*sph;
    invW = pinv(W);
    u = W * FMRI.data;
    data2 = invW * u;
    cnt = cnt+1;
    comp_idx(cnt) = i;
    v_idx(cnt) = var(FMRI.data-data2)/var(FMRI.data);
  end
  plot(comp_idx,v_idx);
  hold on
  plot(comp_idx,v_idx,'g*');
  
  ii = find(v_idx < residual_var);
  lowerstep = round(-0.01*size(FMRI.data,1));
  if abs(lowerstep) < 1,
    lowerstep = -1;
  end
  for i=comp_idx(ii):-1:10,
    [w,sph] = binica(FMRI.data,'pca',i,'maxsteps',0);
    W = w*sph;
    invW = pinv(W);
    u = W * FMRI.data;
    data2 = invW * u;
    vv = var(FMRI.data-data2)/var(FMRI.data);
    if vv > residual_var,
      break;
    end
    
    delete binica*
    delete bias*
  end
  
  hold on
  plot(i+2,vv,'ro');
  a = get(gca,'ylabel');
  set(a,'string','Residule Variance',...
	'fontsize',14,...
	'fontweight','bold');
  a = get(gca,'xlabel');
  set(a,'string','Model Number',...
	'fontsize',14,...
	'fontweight','bold');
  set(gca,'fontsize',14);
  
  txt = sprintf('comp = %d',i+1);
  text(i+1,2*vv,txt);
  comp_no = i+1;
  
  ti = title('Optimal Model Number with S.V.D.');
  set(ti,'fontweight','bold');
  
  delete binica*
  delete bias*
end
  