function ref = fmrlab_import_timing(fname, tscale, tdelay, ev_type, hrf_temp, ...
                                    ref_note, ref_app)

% Inputs:
%    fname    - file name containing the timing info
%    tscale   - time scale of the timing info (can be 'scan', 'sec', 'ms')
%    tdelay   - any time delay applied to input timing (must in 'tscale' unit)
%    ev_type  - event type to create reference function ('event', block')
%    hrf_temp - HRF template (can be 'none', 'gamma', '2gamma', etc.)
%    ref_note - note for the reference time course (e.g., 'button press')
%    ref_app  - append/overwrite the existing reference time course
%
% Outputs:
%    ref - reference function according the input timing and selected HRF template

if nargin < 1,
  [fname, pname] = uigetfile({'*.txt', 'Event timing'}, 'Get Event Timing Info');
  if fname == 0,
    return;
  end
  fname = [pname fname];
end

% read in the timing info
fp = fopen(fname, 'r');
tt = fscanf(fp,'%f');
fclose(fp);

% ask for other info if not provided
if nargin < 7,
  prompt = {'Time scale (scan, sec, ms):',...
            'Time delay (in ''tscale'' unit):',...
            'Event type (event, block):',...
            'HRF template (none, gamma, 2gamma):',...
            'Note:',...
            'Append (yes/no):'};
  ti = 'Import Event Timing';
  defs = {'ms', '0', 'block', '2gamma','','no'};
  a = fmrlab_inputdlg(prompt, ti, 1, defs);
  tscale = a(1);
  tdelay = str2num(char(a(2)));
  ev_type = a(3);
  hrf_temp = a(4);
  ref_note = a(5);
  ref_app = a(6);
end

global FMRI

% convert input timing to scan
if strcmp(tscale,'sec') == 1
  tt = round((1+tt+tdelay)/FMRI.tr);
end
if strcmp(tscale,'ms') == 1
  tt = round((1000+tt+tdelay)/FMRI.tr/1000);
end

% create reference function
ref = zeros(1,min(size(FMRI.data)));

% create block type reference if ev_type == 'block'
if strcmp(ev_type,'block') == 1,
  % check if the length of tt can be exactly divided by 2
  % if not turn this variable to 'event' and create event type reference
  if mod(length(tt),2) ~= 0,
    ev_type = 'event';
  else
    for i=1:2:length(tt),
      ref(tt(i):tt(i+1)) = 1;
    end
  end
end
  
% create event-related type reference if ev_type == 'event'
if strcmp(ev_type,'event') == 1,
  ref(tt) = 1;
end
  
% create HRF template
%figure
%subplot(3,1,1);
%plot(ref);
if strcmp(hrf_temp,'gamma') == 1,
  hrf = gampdf(1:10,8,.5);
  ref = conv(ref,hrf);
  ref = ref(1:min(size(FMRI.data)));
end
if strcmp(hrf_temp,'2gamma') == 1,
  hrf = gampdf(1:10,8,.5)-.5*gampdf(1:10,8,1);
  ref = conv(ref,hrf);
  ref = ref(1:min(size(FMRI.data)));
end
%subplot(3,1,2);
%plot(hrf);
%subplot(3,1,3);
%plot(ref);

% conver the existing reference time course to cell array for 
% backward compatibility
if ~iscell(FMRI.ref),
  FMRI.ref = {FMRI.ref};
  FMRI.refnote{1} = '';
end

if isempty(FMRI.ref),
  FMRI.ref = {ref};
  FMRI.refnote{1} = ref_note;
else
  if strcmp(ref_app,'no') == 1,
    FMRI.ref = [];
    FMRI.ref = {ref};
    FMRI.refnote = [];
    FMRI.refnote{1} = ref_note;
  else
    cc = size(FMRI.ref,2)+1;
    FMRI.ref{cc} = ref;
    FMRI.refnote{cc} = ref_note;
  end
end

