% fmrlab_plot_causality()
%   plot result of granger causality measures obtained
%   by program fmrlab_granger_causality()
%
% Inputs:
%  GCin - granger causality measure
%  FSin - F-test input
%  uni  - unidirectional (1) or bidirectional (0: default)
%  comp_list - component list (default = all components)
%  alpha - p-value threshold (default = 0.01);
%  mcompare - adjust for multiple comparison correction
%             (1/0, default = 1)
%
% Outputs:
%
% Author: Jeng-Ren Duann, 2006-07-05
%

function fmrlab_plot_causality(GCin,FSin,uni,comp_list,alpha,mcompare)

if nargin < 2,
  disp('Usage: fmrlab_plot_causality(GCin,FSin,comp_list,alpha,mcompare);');
end

if nargin < 3,
  uni = 0;
end

if nargin < 4,
  comp_list = 1:size(GCin,1);
end

if nargin < 5,
  alpha = 0.01;
end

if nargin < 6,
  mcompare = 1;
end

if comp_list == 0,
  comp_list = 1:size(GCin,1);
end

sz = size(GCin);
GC = zeros(sz);
FS = zeros(sz);
if mcompare == 1,
  alpha = alpha./((sz(1)*(sz(1)-1))/2);
end

for ii=1:sz(1),
  for jj=1:sz(2),
    if GCin(ii,jj) < alpha,
      GC(ii,jj) = 1;
      FS(ii,jj) = FSin(ii,jj);
    end
  end
end

% mark the ones with uni-directional causality with 0.33
R = GC.*GC';
idx1 = find(GC);
idx2 = find(R);
idx = setdiff(idx1,idx2);
GC(idx) = 0.33;

% plot nodes
GC = GC(comp_list,comp_list);
FS = FS(comp_list,comp_list);
sz = size(GC);
set(gca,'xticklabel',[]);
delta = 2*pi/sz(1);
x = zeros(sz(1),1);
y = zeros(sz(1),1);
for ii=1:sz(1),
  x(ii) = cos(delta*(ii-1));
  y(ii) = sin(delta*(ii-1));
end
for ii=1:sz(1),
  h = plot(x(ii),y(ii),'.');
  set(h,'color','k','markersize',20);
  hold on
end

% link nodes with uni-directional causality
[xx,yy]=find(GC);
uni = [xx yy];
temp = GC.*GC;
[toidx,fromidx]=find(temp==1);
bi = [toidx fromidx];
uni = setdiff(uni,bi,'rows');
toidx = uni(:,1);
fromidx = uni(:,2);
clr = colormap(jet(round(max(FS(:)))));
%xt=[1 -sin(pi/6) -sin(pi/6)];
%yt=[0 cos(pi/6) -cos(pi/6)];
for ii=1:length(toidx),
  x1 = x(fromidx(ii));
  y1 = y(fromidx(ii));
  x2 = x(toidx(ii));
  y2 = y(toidx(ii));
  %h = line([x1 x2],[y1 y2]);
  %set(h,'color','b');

  % plot triangle
  % determine the angle of the triangle
  m = ((y2-y1)/2)/((x2-x1)/2);
  if x2 > x1
    p = atan(m);
  else
    p = atan(m)+pi;
  end
  delta = 0.015/dist([x1 y1],[x2 y2]');
  x11 = (x1-x2)*cos(-delta) - (y1-y2)*sin(-delta) + x2;
  y11 = (x1-x2)*sin(-delta) + (y1-y2)*cos(-delta) + y2;
  x12 = (x1-x2)*cos(delta) - (y1-y2)*sin(delta) + x2;
  y12 = (x1-x2)*sin(delta) + (y1-y2)*cos(delta) + y2;
  xd = [x2 x11 x12];
  yd = [y2 y11 y12]; 
  %for jj=1:3,
  %  xd(jj) = cos(p)*xt(jj) - sin(p)*yt(jj);
  %  yd(jj) = sin(p)*xt(jj) + cos(p)*yt(jj);
  %end
  %xd = (xd-cos(p))/20 + (x1+x2)/2;
  %yd = (yd-sin(p))/20 + (y1+y2)/2;
  cc = clr(round(FS(toidx(ii),fromidx(ii))),:);
  patch(xd,yd,cc,'edgecolor',cc);
end

% link nodes with bi-directional causality
if uni == 0,
  temp = GC.*GC;
  [toidx,fromidx]=find(temp==1);
  for ii=1:length(toidx),
    x1 = x(fromidx(ii));
    y1 = y(fromidx(ii));
    x2 = x(toidx(ii));
    y2 = y(toidx(ii));
    h = line([x1 x2],[y1 y2]);
    set(h,'color','r','linewidth',4);
  end
end

% label nodes
xx = 1.1*x;
yy = 1.1*y;
for ii=1:length(comp_list),
  h = text(xx(ii),yy(ii),num2str(comp_list(ii)));
  set(h,'fontsize',8);
end
set(gca,'box','off');
axis off;
axis square;
xlim = [-1.2 1.2];
ylim = [-1.2 1.2];

gcf;
h = cbar;
aa = linspace(0,max(FS(:)),11)';
bb = num2str(aa,'%3.1f');
set(h,'yticklabel',bb);
pos = get(h,'position');
pos(1) = pos(1)-0.1;
set(h,'position',pos);
