% model_selection() - estimate the minmum model no should be used to 
%                     decompose the input data matrix using AIC, BIC
%                     and MDL criteria.
%
% Usage:
%   [mea model_no min_mea] = model_selection(data, [method]);
%
% Input:
%   data   - a 2D data matric to estimate (data have to be prepared in the format of 
%            [time point x voxel no] in fMRI case; for EEG case, please try
%            [channels by time point] instead.
%   method - method to use, can be 'AIC', 'BIC', 'MDL' or 'MDS', default='AIC'
%
% Output:
%   mea     - an 1D array contained all the AIC/MDL measures, (M-1) in size,
%             M is the number of rows of the input data
%   model_no- minimum model number suggested
%   min_mea - minimum AIC/MDL measure corresponging to the selected model no.
%
% Author: Jeng-Ren Duann, PhD, SCCN/INC/UCSD, 2006-01-17
%
% Notes:
%   1. no model_no and min_mea outputs for 'MDS' option.
%
% Revision:
%   2006-02-07 Add option of using multidimensional scaling (MDS). To use MDS option, one has to 
%              have matlab stat toolbox and lot of memory.

function [mea,model_no,min_mea] = model_selection(data,method)

if nargin < 2,
  method = 'AIC';
end

[M,N] = size(data);
mm = mean(data(:));
data = (data-mm);
cv = cov(data');
[s,v,u]=svd(cv);
v = diag(v);
%[e,v] = pcsquash(data);

if strcmp(method,'AIC')==1,
  aic = zeros(1,M-1);
  for i=1:M-1,
    % calculate L(theta)
    l1 = 1;
    for j=i+1:M,
      l1 = l1 * v(j);
    end
    l2 = 0;
    for j=i+1:M,
      l2 = l2 + v(j);
    end
    ll = log(power(l1,1/(M-i)))/(l2/(M-i));
    aic(i) = -2*N*(M-i)*ll + 2*(1+i+(i-1)/2);
  end
  mea = aic;
  ii = find(abs(mea) ~= Inf);
  [a,b] = min(mea(ii));
  model_no = ii(b);
  min_mea = a;
elseif strcmp(method,'BIC')==1,
  bic = zeros(1,M-1);
  for i=1:M-1,
    % calculate L(theta)
    l1 = 1;
    for j=i+1:M,
      l1 = l1 * v(j);
    end
    l2 = 0;
    for j=i+1:M,
      l2 = l2 + v(j);
    end
    ll = log(1+power(l1,1/(M-i)))/(l2/(M-i));
    bic(i) = -2*N*(M-i)*ll + log(N)*(1+i+(i-1)/2);
  end
  mea = bic;
  ii = find(abs(mea) ~= Inf);
  [a,b] = min(mea(ii));
  model_no = ii(b);
  min_mea = a;
elseif strcmp(method,'MDL')==1,
  mdl = zeros(1,M-1);
  for i=1:M-1,
    % calculate L(theta)
    l1 = 1;
    for j=i+1:M,
      l1 = l1 * v(j);
    end
    l2 = 0;
    for j=i+1:M,
      l2 = l2 + v(j);
    end
    ll = log(power(l1,1/(M-i)))/(l2/(M-i));
    mdl(i) = -1*N*(M-i)*ll + 0.5*log(N)*(1+i+(i-1)/2);
  end
  mea = mdl;
  ii = find(abs(mea) ~= Inf);
  [a,b] = min(mea(ii));
  model_no = ii(b);
  min_mea = a;
else
  d = pdist(data');
  [y,e] = cmdscale(d);
  mea = [];
  for i=1:size(data,1);
    mea = [mea max(abs(d-pdist(y(:,1:i))))];
  end
end


