% mpica - run stand-alone parallel binary ICA application mpica by Jason Palmer.
%         If data are stored in a binary float file, they are not read into Matlab.
% Usage:
%  >> [wts,sph,A,b,p,q,cx,ck,mn,compvar] = mpica( datavar, 'key1', value1, ...
%                                                           'key2', value2 ...);
%  >> [wts,sph,A,b,p,q,cx,ck,mn,compvar] = mpica('datafile', chans, frames, ...
%                                                           'key1', value1, ...);
%
% Inputs:
%   datavar  = (chans,frames) data matrix. Data nchans & nframes read from datavar.
%  % else
%   datafile       = ('filename') string name of binary floating-point data file 
%                    (multiplexed by channel). Requires chans & frames below.
%   chans          = number of channels in data matrix
%   frames         = number of time points in data matrix
%
% Optional inputs: (with = defaults)
%   processors     = 1        number of execution nodes (each may have multiple CPUs)
%   max_iter       = 100      maximum number of iterations to perform
%   block_size     = 6000     learning block size, will be divided over available 
%                             processors
%   alpha          = 0.01     initial alpha (learning rate)
%   alphafact      = 0.9      annealing rate - factor by which to decrease alpha when 
%                             likelihood decreases, or at each step if 'doreject', 0.
%   minalpha       = 1.0e-12  stop when alpha (learning rate) becomes smaller than this
%   gradtype       = 5        component density model used in the training:
%                             0 = cosh; 1 = logistic; 2 = laplacian 
%                             3 = quasi-logistic (generalized gaussian with p=1.5), 
%                             4 = generalized gaussian (p learned for each component), 
%                             5 = pq: asymmetric generalized gaussian (generalized 
%                                 gaussian with different p for positive and negative
%                                 values wrt the component bias (mean) value.
% Generalized gaussian model:
%   getcost        = 1        0 = use alphafact as annealing ignore likelihood
%                             1 = compute data likelihood; anneal when likelihood 
%                                 has stopped increasing 
%   p0             = 1.7      in pq decomposition, starting value for p (and q) 
%   lowp           = 1.2      in pq decomposition, min value p (and q) can adapt to 
%
% Perform online data rejection:
%   doreject       = 1        0 = don't reject data based on likelihood
%                             1 = do reject unlikely data 
%   firstreject    = 50        iteration number on which to perform first rejection
%   rejectstep     = 20       number iterations between rejections after firstreject
%   maxreject      = 5        maximum number of times to perform rejection
%   devfact        = 3.0      rejection threshold: number of standard deviations below 
%                             the mean data likelihood to begin rejecting data points
%
% Specify learning rate for dimension p:
%   alphap0        = 0.1      initial value for p-adaptation rate (alphap)
%                             in generalized gaussian and pq gradtypes
%   maxalphap      = 0.5      maximum p-adaptation rate (alphap)
%                             
% Continue training: 
%   W_infile       = none     file containing initial weight values 
%   rej_infile     = none     file containing vector of length ntot 
%                             with zeros for rejected frames
%  Note: To continue training, re-start the algorithm in the same state as at 
%        the end of previous training.  State variables include: learning rate, 
%        p (and possibly q) values, and the weight and rejection matrices.
%        The 'Wbp' output file (see below) contains the weight, p and q matrices 
%        from the previous run; the 'ck' output file contains the rejection matrix. 
%        Thus, setting W_infile   = (full path filename of the Wbp file, and 
%                      rej_infile = (full path filename of the ck file, and
%                      alpha      = ending learning rate in previous run,
%        mpica will then resume the previous training. 
%
% Outputs:
%   wts         (ncomps,ncomps) unmixing weight matrix
%   sph         (ncomps,ncomps) unmixing sphere matrix
%                (full unmixing matrix is wts*sph)
%   A           (ncomps,ncomps) components A = inv(wts*sph)
%   b           (ncomps,1) activation bias vector
%   p           (ncomps,1) p dimension vector
%   q           (ncomps,1) q dimension vector (if pq gradtype)
%   cx          (n_iterations,1) log likelihood at each iteration
%   ck          (n_time_frames,1) log likelihood of each time point 
%               in the last iteration
%   mn          (ncomps,1) the means of the original data channels 
%               removed before training
%   compvar     (ncomps,1) the variance in the original data 
%               accounted for by each component 
%
% Output files (created in a randomly-named temp folder in binary floating-point):
%   A           components written in a (1, ncomps^2) vector A250, A500, ...           
%               components saved every 250 iterations (same format as output)
%   S           unmixing sphere matrix written as a (1, ncomps^2) vector
%   Wbp         unmixing weight matrix (1, ncomps^2) vector + (1, ncomps) 
%               activation bias vector + (1,ncomps) p vector + (for gradtype 
%               pq only)  (1,ncomps) q vector
%   compvar     (1,ncomps) component variances in the original data
%   mean        (1,ncomps) means of the original data removed before training
%   out.txt     record of console output of the algorithm in text file format
%
% Author: Bigdely Shamlo for parallel mpica by Jason Palmer, SCCN/INC/UCSD 10/05
%         after binica(), by Scott Makeig, that called the binary version 
%         (by Sigurd Enghoff) of runica() by Makeig, Bell, Lee, Jung, et al. 
% See also: binica(), runica()
%123456789012345678901234567890123456789012345678901234567890123456789012

% Copyright (C) 2005 Scott Makeig, SCCN/INC/UCSD, scott@sccn.ucsd.edu
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

% $Log: binica.m,v $
% Revision 1.13  2005/03/14 19:43:45  arno
% fixing argument passing
%
% Revision 1.12  2005/03/13 19:03:45  scott
% trying to read flag,arg pairs - FAILS!!!
% Arno/Hilit - PLEASE CHECK LINE 219 and etc. ! -Scott
%
% Revision 1.11  2005/03/13 17:14:40  peter
% comments
%
% Revision 1.10  2004/08/19 19:10:41  arno
% did not change anything
%
% Revision 1.9  2004/02/28 00:00:22  arthur
% edit commandline output text
%
% Revision 1.8  2003/10/25 23:52:05  arno
% remove asking for permission to remove file
%
% Revision 1.7  2003/10/07 18:55:03  arno
% undo changes
%
% Revision 1.6  2003/10/07 18:54:30  arno
% testing
%
% Revision 1.5  2002/11/15 01:44:18  arno
% header for web
%
% Revision 1.4  2002/08/05 18:17:06  arno
% debugging directory finding
%
% Revision 1.3  2002/06/25 02:38:49  scott
% calrified error msgs -sm
%
% Revision 1.2  2002/05/01 18:22:36  arno
% making binica available from everywhere
%
% Revision 1.1  2002/04/05 17:36:45  jorn
% Initial revision
%

% 08/07/00 Added warning to update icadefs.m -sm
% 09/08/00 Added tmpint to script, weights and sphere files to avoid
%          conflicts when multiple binica sessions run in the same pwd -sm
% 10/07/00 Fixed bug in reading wts when 'pca' ncomps < nchans -sm
% 07/18/01 replaced var ICA with ICABINARY to try to avoid Matlab6 bug -sm
% 11/06/01 add absolute path of files (lines 157-170 & 198) -ad
% 01-25-02 reformated help & license, added links -ad 
 
function [wts,sph,A,b,p,q,cx,ck,mn,compvar] = mpica(data,var2,var3,var4,var5,var6,var7,var8,var9,var10,var11,var12,var13,var14,var15,var16,var17,var18,var19,var20,var21,var22,var23,var24,var25)

if nargin < 1 | nargin > 25
    more on
    help mpica
    more off
    return
end

fmrlab_icadefs %for future when we include MPRUN and MPICABINARY in fmrlab_icadefs
%MPRUN = '/opt/mpich/mpich-1.2.7/bin/mpirun -nolocal -machinefile /home/nima/mica/machines'; % Location and options for mpirun command
MPRUN = '/opt/mpich/mpich-1.2.7/bin/mpirun -nolocal -machinefile /home/duann/matlab/fmrlab3.0/MPICA/machines'; % Location and options for mpirun command
MPICABINARY = '/home/duann/matlab/fmrlab3.0/mpica12';
%MPICABINARY = '/home/nima/mica/mpica12';
SC = 'mpica.sc'; % Nima: master script for mpica

if ~exist('SC')
  fprintf('mpica(): You need to update your fmrlab_icadefs file to include ICABINARY and SC.\n')
  return
end;
if exist(SC) ~= 2
  fprintf('mpica(): No ica source file ''%s'' is in your Matlab path, check...\n', SC);
  return
else
	SC = which(SC);
	fprintf('mpica: using source file ''%s''\n',  SC);
end
if exist(ICABINARY) ~= 2
  fprintf('mpica(): ica binary ''%s'' is not in your Matlab path, check\n', ICABINARY);
  return
else
	ICABINARYdir = which(MPICABINARY);
	if ~isempty(ICABINARYdir)
		fprintf('mpica(): using binary mpica file ''%s''\n', ICABINARYdir);
	else
		fprintf('mpica(): using binary mpica file ''\?/%s''\n', MPICABINARY);
	end;
end

%
% select random integer 1-10000 to index the mpica data files
% make sure no such script file already exists in the pwd
%
scriptfile = SC;
while exist(scriptfile)
  tmpint = randperm(10000);
  tmpint = int2str(tmpint(10000));
  scriptfile = ['mpica' tmpint '.sc'];
end

nchans = 0;
tmpdata = [];
if ~isstr(data) % data variable name given
  if ~exist('data')
    fprintf('\nmpica(): Variable name data not found.\n');
    return
  end
  nchans = size(data,1);
  nframes = size(data,2);
  tmpdata = ['mpica' tmpint '.fdt'];
  floatwrite(data,tmpdata);
  datafile = tmpdata;
  firstarg = 2;

else % data filename given
  if ~exist(data)
    fprintf('\nmpica(): File data not found.\n')
    return
  end
  datafile = data;
  if nargin<3
    fprintf(...
'\nmpica(): Data file name must be followed by chans, frames\n');
    return
  end
  nchans = var2;
  nframes = var3;
  if isstr(nchans) | isstr(nframes)
    fprintf(...
'\nmpica(): chans, frames args must be given after data file name\n');
    return
  end
  firstarg = 4;
end
%
% read in the master ica script file SC
%
flags = [];
args  = [];
fid = fopen(SC,'r');
if fid < 0
  fprintf('\nmpica(): Master .sc file %s not read!\n',SC)
     return
end


%
% read SC file info into flags and args lists
%
s = [];
f = 0; % flag number in file
while isempty(s) | s ~= -1
 s = fgetl(fid);
  if s ~= -1
   if ~isempty(s)
     s = rmcomment(s,'#');
     if ~isempty(s)
       f = f+1;
       s = rmspace(s);
       [w s]=firstword(s);
       if ~isempty(s)
          flags{f} = w;
          s = rmspace(s);
          [w s]=firstword(s);
          args{f} = w;
       end
     end
   end
  end
end 

%
% substitute the flags/args pairs in the .sc file
%
arg = firstarg;
if arg > nargin
   fprintf('\nmpica(): no optional (flag, argument) pairs received.\n');
else
 if (nargin-arg+1)/2 > 1
    fprintf('\nmpica(): processing %d (flag, arg) pairs.\n',(nargin-arg+1)/2);
 else
    fprintf('\nmpica(): processing one (flag, arg) pair.\n');
 end
 while arg <= nargin

  eval(['OPTIONFLAG = var' int2str(arg) ';']); % WHY DOES THIS MAKE FLAG (64,3) ?????  
  % fprintf('WARNING: mpica() did NOT read the specified flag, arg  % pair!!!!\n'); 

  if arg == nargin
    fprintf('\nmpica(): Flag %s needs an argument.\n',OPTIONFLAG)
    return
  end
  eval(['Arg = var' int2str(arg+1) ';']);

  if strcmpi(OPTIONFLAG,'pca')
        ncomps = Arg; % get number of components out for reading wts.
  end
  arg = arg+2;

  nflags = f;
  for f=1:length(flags)   % replace arg with Arg
    if strcmp(OPTIONFLAG,flags{f})
       args{f} = num2str(Arg);
       fprintf('setting %s, %s.\n',flags{f},args{f});
    end
  end
 end
end

%
% insert name of data files, chans and frames
%
for x=1:length(flags)
  if strcmp(flags{x},'x_infile')
     datafile = [pwd '/' datafile];
     args{x} = datafile;
  elseif strcmp(flags{x},'outdir') 
     outdirectory = [pwd '/' 'mpica' tmpint];
     args{x} = outdirectory;
     weightsfile = [outdirectory '/' 'Wbp'];
     spherefile = [outdirectory '/' 'S'];
  elseif strcmp(flags{x},'nw') % Write the number of channels
     args{x} = int2str(nchans);
elseif strcmp(flags{x},'ntot') % Write the number of frames (samples)
     args{x} = int2str(nframes);
  end
end


%
% write the new .sc file
%
fid = fopen(scriptfile,'w');
for x=1:length(flags)
  fprintf(fid,'%s %s\n',flags{x},args{x});
end
fclose(fid);
if ~exist(scriptfile)
  fprintf('\nmpica(): ica script file %s not written.\n',...
                                   scriptfile);
  return
end
 

% Find the number of processors 
for x = 1:length(flags)
    if strcmp(flags{x},'processors')
        processors = args{x}; % processors contains the number of processors to run on as a string
    end
end    


%
% %%%%%%%%%%%%%%%%%%%%%% run ica %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
   fprintf('\nRunning ica from script file %s\n',scriptfile);
   if exist('ncomps')
        fprintf('   Finding %d components.\n',ncomps);
   end
   %eval('!ssh juggling');
   eval_call = ['!ssh juggling ' MPRUN  ' -np ' processors ' ' MPICABINARY ' ' pwd '/' scriptfile]
   eval(eval_call); %just for test 
%  eval_call = [MPRUN  ' -np ' processors ' ' pwd '/' 'mpica9 ' pwd '/' scriptfile]
%  qsub(eval_call, 'mpica command');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%


%
% read in wts and sph results.
%
if ~exist('ncomps')
  ncomps = nchans;
end

% wts = floatread(weightsfile,[ncomps Inf]);
% sph = floatread(spherefile,[nchans Inf]);

[wts,sph,A,b,p,q,cx,ck,mn,compvar] = geticaout(outdirectory);

if isempty(wts)
   fprintf('\nmpica(): weight matrix not read.\n')
   return
end


if isempty(sph)
   fprintf('\nmpica():  sphere matrix not read.\n')
   return
end
fprintf('\nica files left in pwd:\n');
eval(['!ls -l ' scriptfile ' ' weightsfile ' ' spherefile]);
fprintf('\n');

if isstr(data)
  whos wts sph
else
  whos data wts sph
end

[wts A]=sort_comps(wts,sph,data);

%
% If created by mpica(), rm temporary data file
% Note: doesn't remove the .sc .wts and .fdt files
if ~isempty(tmpdata)
    eval(['!rm -f ' datafile]);
%    eval(['!rm -f *fdt']);
end

%
%%%%%%%%%%%%%%%%%%% included functions %%%%%%%%%%%%%%%%%%%%%%
%
function sout = rmcomment(s,symb)
     n =1;
     while (s(n)~=symb) % discard # comments
        n = n+1;
     end
     if n == 1
        sout = [];
     else
       sout = s(1:n-1);
     end
    
function sout = rmspace(s)
       n=1;          % discard leading whitespace
       while n<length(s) & isspace(s(n))
          n = n+1;
       end
       if n<length(s)
          sout = s(n:end);
       else
          sout = [];
       end

function [word,sout] = firstword(s)
       n=1;         
       while n<=length(s) & ~isspace(s(n))
          n = n+1;
       end
       if n>length(s)
         word = [];
         sout = s;
       else
         word = s(1:n-1);
         sout = s(n:end);
       end
